/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "Time.hh"
#include <string>

/**  The %frame_name class construct a frame name in the canonical format, 
  *  i.e. \e \<prefix>-\<gps>-\<dt>.\<ext>
  */
class frame_name {
public:
    /**  Default constructor. By default the extension is ".gwf" the frame 
      *	 length is 1s and the maximum dpepth is 1.
      */
    frame_name(void);

    /**  Construct a frame nae instance. 
     */
    frame_name(const std::string& dir_pat, const std::string& pfx_pat, int dt=1,
	       const std::string& ext=std::string("gwf"));

    /**  Get the directory name for the specified time. The directory name is
      *  constructedfrom the directory parrtern string and the time arguemnt.
      */
    std::string dir_name(const Time& t) const;

    /** DIrectory pattern string.
     */
    const std::string& directory(void) const;

    /** Frame file extension string.
     */
    const std::string& extension(void) const;

    /**  Complete file path for the specified time and frame length.
      *  \brief Generate frame file path.
      *  \param t Gps time of frame start.
      *  \param dt Frame length in seconds
      *  \return Frame file path string.
      */
    std::string file_path(const Time& t, int dt=0) const;

    /**  Make the specified directory. Missing parent directories are will be 
      *  constructed to the specified depth. For example, if the immediate 
      *  parent  of the specified directory is missing and the depth is at 
      *  least 2, the missing parent will be created. All created directories
      *  will be assigned the file access mode specified by \a mode.
      *  \brief Construct a directory.
      *  \param dir   Directory path.
      *  \param depth Maximum number of directories to be created.
      *  \param mode  Access mode of the directory(ies) created.
      *  \return True if the specified directory exists on return.
      */
    bool make_dir(const std::string& dir, int depth=1, int mode=0775);

    /**  Return the frame file name prefix string.
     */
    const std::string& prefix(void) const;

    /**  Set the directory path pattern The directory name(s) are constructed 
      *  by replacing escape sequences with values derived from the frame start
      *  time.
      */
    void set_directory(const std::string& dir_pat);

    /**  Set the maximum number of new directories to create.
      *  \brief directory creation limit.
      *  \param deep Maxumium directory count.
      */
    void set_depth(int deep);

    /** Set the default frame length.
     */
    void set_dt(int dt);

    /**  Set the extension string. The default value is "gwf" and should not be
      *  changed for frame files.
      *  \brief Set extension.
      *  \param ext Extension string.
      */ 
    void set_extension(const std::string& ext);

    /**  Set the frame file name prefix. THe prefix includes both the 
      *  observatory identifier(s) and the frame typre code.
      *  \brief Set Frame file prefix.
      *  \param ext Prefix string.
      */ 
    void set_prefix(const std::string& pfx);

    /**  Generate frame temporary path for the specified time and frame length.
      *  \brief Generate temporary file path.
      *  \param t Gps time of frame start.
      *  \param dt Frame length in seconds
      *  \return Frame file path string.
      */
    std::string temp_path(const Time& t, int dt=0) const;

private:
    std::string _dir;
    std::string _prefx;
    std::string _ext;
    int         _dt;
    int         _deep;
    std::string _lastDir;
};

//======================================  Inline accessors
inline const std::string&
frame_name::directory(void) const {
    return _dir;
}

inline const std::string&
frame_name::extension(void) const {
    return _ext;
}

inline const std::string&
frame_name::prefix(void) const {
    return _prefx;
}
